@@ -1,5 +1,5 @@
 # class definitions
-# 
+#
 # initial version: YR 25/03/2009
 # added ModelSyntax: YR 02/08/2010
 # deleted ModelSyntax: YR 01/11/2010 (using flattened model syntax now)
@@ -50,7 +50,7 @@ setClass("lavSampleStats",         # sample moments
         res.var="list",            # residual variances
         res.th="list",             # residual thresholds
         res.th.nox="list",         # residual thresholds ignoring x
-        res.slopes="list",         # slopes exo (if conditional.x) 
+        res.slopes="list",         # slopes exo (if conditional.x)
         res.int="list",            # intercepts (if conditional.x)
 
         mean.x="list",             # mean exo
@@ -102,6 +102,7 @@ setClass("lavModel",          # MATRIX representation of the sem model
 
         nblocks="integer",
         ngroups="integer",   # only for rsem!! (which uses rsem:::computeDelta)
+        nefa="integer",
         nmat="integer",
         nvar="integer",
         num.idx="list",
@@ -120,7 +121,7 @@ setClass("lavModel",          # MATRIX representation of the sem model
         x.def.idx="integer",
         x.ceq.idx="integer",
         x.cin.idx="integer",
-        #x.free.var.idx="integer",
+        x.free.var.idx="integer",
 
         eq.constraints="logical",
         eq.constraints.K="matrix",
@@ -139,6 +140,7 @@ setClass("lavModel",          # MATRIX representation of the sem model
         cin.rhs="numeric",
         cin.linear.idx="integer",
         cin.nonlinear.idx="integer",
+        ceq.efa.JAC="matrix",
         con.jac="matrix",
         con.lambda="numeric",
 
@@ -151,6 +153,11 @@ setClass("lavModel",          # MATRIX representation of the sem model
         ov.y.dummy.ov.idx="list",
         ov.y.dummy.lv.idx="list",
 
+        ov.efa.idx="list",
+        lv.efa.idx="list",
+        H="list",
+        lv.order="list",
+
         estimator="character"
     )
 )
@@ -191,7 +198,7 @@ setClass("lavaan",
         SampleStats = "lavSampleStats",  # sample statistics
         Model       = "lavModel",        # internal matrix representation
         Cache       = "list",            # housekeeping stuff
-        Fit         = "Fit",             # fitted results 
+        Fit         = "Fit",             # fitted results
         boot        = "list",            # bootstrap results
         optim       = "list",            # optimizer results
         loglik      = "list",            # loglik values and info
@@ -201,7 +208,7 @@ setClass("lavaan",
         h1          = "list",            # unrestricted model results
         baseline    = "list",            # baseline model results
         external    = "list"             # optional slot, for add-on packages
-    ) 
+    )
 )
 
 setClass("lavaanList",
@@ -223,6 +230,8 @@ setClass("lavaanList",
         testList        = "list",
         optimList       = "list",
         impliedList     = "list",
+        h1List          = "list",
+        loglikList      = "list",
         funList         = "list",
         external        = "list"       # optional slot, for add-on packages
     )
@@ -234,4 +243,4 @@ setClass("lavaanList",
 
 
 
-                      
+
