@@ -12,6 +12,7 @@
 # - 14 Jan 2014: merge 02lavaanUser.R with lav_partable.R
 #                move syntax-based code to lav_syntax.R
 # - 26 April 2016: handle multiple 'blocks' (levels, classes, groups, ...)
+# - 24 March 2019: handle efa sets
 
 lavaanify <- lavParTable <- function(
 
@@ -20,7 +21,11 @@ lavaanify <- lavParTable <- function(
                       int.ov.free      = FALSE,
                       int.lv.free      = FALSE,
                       orthogonal       = FALSE,
+                      orthogonal.y     = FALSE,
+                      orthogonal.x     = FALSE,
+                      orthogonal.efa   = FALSE,
                       std.lv           = FALSE,
+                      effect.coding    = "",
                       conditional.x    = FALSE,
                       fixed.x          = TRUE,
                       parameterization = "delta",
@@ -35,6 +40,7 @@ lavaanify <- lavParTable <- function(
                       auto.cov.y       = FALSE,
                       auto.th          = FALSE,
                       auto.delta       = FALSE,
+                      auto.efa         = FALSE,
 
                       varTable         = NULL,
                       ngroups          = 1L,
@@ -66,7 +72,7 @@ lavaanify <- lavParTable <- function(
     CON  <- attr(FLAT, "constraints"); attr(FLAT, "constraints") <- NULL
 
     # extra constraints?
-    if(!is.null(constraints) && nchar(constraints) > 0L) {
+    if(!is.null(constraints) && any(nchar(constraints) > 0L)) {
         FLAT2 <- lavParseModelString(model.syntax=constraints, warn=warn)
         CON2 <- attr(FLAT2, "constraints"); rm(FLAT2)
         CON <- c(CON, CON2)
@@ -107,6 +113,7 @@ lavaanify <- lavParTable <- function(
             auto.cov.y      = TRUE
             auto.th         = TRUE
             auto.delta      = TRUE
+            auto.efa        = TRUE
         } else
 
         if(model.type == "growth") {
@@ -120,6 +127,7 @@ lavaanify <- lavParTable <- function(
             auto.cov.y      = TRUE
             auto.th         = TRUE
             auto.delta      = TRUE
+            auto.efa        = TRUE
         }
     }
 
@@ -194,14 +202,16 @@ lavaanify <- lavParTable <- function(
                 block.id = block.id,
                 meanstructure = meanstructure,
                 int.ov.free = int.ov.free, int.lv.free = int.lv.free,
-                orthogonal = orthogonal, std.lv = std.lv,
+                orthogonal = orthogonal, orthogonal.y = orthogonal.y,
+                orthogonal.x = orthogonal.x, orthogonal.efa = orthogonal.efa,
+                std.lv = std.lv,
                 conditional.x = conditional.x, fixed.x = fixed.x,
                 parameterization = parameterization,
                 auto.fix.first = auto.fix.first,
                 auto.fix.single = auto.fix.single,
                 auto.var = auto.var, auto.cov.lv.x = auto.cov.lv.x,
                 auto.cov.y = auto.cov.y, auto.th = auto.th,
-                auto.delta = auto.delta,
+                auto.delta = auto.delta, auto.efa = auto.efa,
                 varTable = varTable, group.equal = NULL,
                 group.w.free = group.w.free, ngroups = 1L)
             LIST.block <- as.data.frame(LIST.block, stringsAsFactors = FALSE)
@@ -237,13 +247,15 @@ lavaanify <- lavParTable <- function(
         LIST <- lav_partable_flat(FLAT, blocks = "group",
             meanstructure = meanstructure,
             int.ov.free = int.ov.free, int.lv.free = int.lv.free,
-            orthogonal = orthogonal, std.lv = std.lv,
+            orthogonal = orthogonal, orthogonal.y = orthogonal.y,
+            orthogonal.x = orthogonal.x, orthogonal.efa = orthogonal.efa,
+            std.lv = std.lv,
             conditional.x = conditional.x, fixed.x = fixed.x,
             parameterization = parameterization,
             auto.fix.first = auto.fix.first, auto.fix.single = auto.fix.single,
             auto.var = auto.var, auto.cov.lv.x = auto.cov.lv.x,
             auto.cov.y = auto.cov.y, auto.th = auto.th,
-            auto.delta = auto.delta,
+            auto.delta = auto.delta, auto.efa = auto.efa,
             varTable = varTable, group.equal = group.equal,
             group.w.free = group.w.free,
             ngroups = ngroups)
@@ -310,6 +322,7 @@ lavaanify <- lavParTable <- function(
             MOD.upper <- MOD[[el]]$upper
             MOD.label <- MOD[[el]]$label
             MOD.prior <- MOD[[el]]$prior
+            MOD.efa   <- MOD[[el]]$efa
 
             # check for single argument if multiple groups
             if(ngroups > 1L && length(idx) > 1L) {
@@ -320,11 +333,13 @@ lavaanify <- lavParTable <- function(
                 if(length(MOD.lower) == 1L) MOD.lower <- rep(MOD.lower, ngroups)
                 if(length(MOD.upper) == 1L) MOD.upper <- rep(MOD.upper, ngroups)
                 if(length(MOD.prior) == 1L) MOD.prior <- rep(MOD.prior, ngroups)
+                if(length(MOD.efa)   == 1L) MOD.efa   <- rep(MOD.efa,   ngroups)
                 # B) here we do NOT! otherwise, it would imply an equality
                 #                    constraint...
                 #    except if group.equal="loadings"!
                 if(length(MOD.label) == 1L) {
-                    if("loadings" %in% group.equal) {
+                    if("loadings" %in% group.equal ||
+                       "composite.loadings" %in% group.equal) {
                         MOD.label <- rep(MOD.label, ngroups)
                     } else {
                         MOD.label <- c(MOD.label, rep("", (ngroups-1L)) )
@@ -339,6 +354,7 @@ lavaanify <- lavParTable <- function(
                 (!is.null(MOD.lower) && nidx != length(MOD.lower)) ||
                 (!is.null(MOD.upper) && nidx != length(MOD.upper)) ||
                 (!is.null(MOD.prior) && nidx != length(MOD.prior)) ||
+                (!is.null(MOD.efa)   && nidx != length(MOD.efa))   ||
                 (!is.null(MOD.label) && nidx != length(MOD.label)) ) {
                 el.idx <- which(LIST$mod.idx == el)[1L]
                 stop("lavaan ERROR: wrong number of arguments in modifier (",
@@ -370,6 +386,13 @@ lavaanify <- lavParTable <- function(
                 }
                 LIST$prior[idx] <- MOD.prior
             }
+            if(!is.null(MOD.efa)) {
+                # do we already have a `efa' column? if not, create one
+                if(is.null(LIST$efa)) {
+                    LIST$efa <- character( length(LIST$lhs) )
+                }
+                LIST$efa[idx] <- MOD.efa
+            }
             if(!is.null(MOD.lower)) {
                 # do we already have a `lower' column? if not, create one
                 if(is.null(LIST$lower)) {
@@ -464,7 +487,7 @@ lavaanify <- lavParTable <- function(
             } else {
             # 2. ref.idx is a free parameter
                 # user-label?
-                #if(nchar(LIST$label[ref.idx])  > 0) {
+                #if(any(nchar(LIST$label[ref.idx])  > 0)) {
                 #    lhs.lab <- LIST$label[ref.idx]
                 #} else {
                 #    lhs.lab <- PLABEL[ref.idx]
@@ -551,6 +574,254 @@ lavaanify <- lavParTable <- function(
     LIST$label[def.idx] <- LIST$lhs[def.idx]
 
 
+    # handle EFA equality constraints
+    # Note: we should also check if they are really needed:
+    #       eg., if all the factor-loadings of the 'second' set (time/group)
+    #       are constrained to be equal to the factor-loadings of the first
+    #       set, no further constraints are needed
+    if(auto.efa && !is.null(LIST$efa)) {
+        # for each set, for each block
+        nblocks <- lav_partable_nblocks(LIST)
+        set.names <- lav_partable_efa_values(LIST)
+        nsets <- length(set.names)
+
+        plabel <- character(0L)
+        for(b in seq_len(nblocks)) {
+            for(s in seq_len(nsets)) {
+                # lv's for this block/set
+                lv.nam.efa <- unique(LIST$lhs[LIST$op == "=~" &
+                                              LIST$block == b &
+                                              LIST$efa == set.names[s]])
+                if(length(lv.nam.efa) == 1L) {
+                    # nothing to do (warn?)
+                    next
+                }
+
+                # equality constraints on ALL factor loadings in this set?
+                # --> no constraints are needed
+                if(b > 1L || s > 1L) {
+                    # collect plabels for this set, if any
+                    set.idx <- which(LIST$op == "=~" &
+                                     LIST$block == b &
+                                     LIST$lhs %in% lv.nam.efa)
+                    plabels.set <- LIST$plabel[ set.idx ]
+
+                    # check simple == equalities
+                    # TODO: what about more complex equality constraints?
+                    eq.idx <- which(LIST$op == "==")
+                    con.labels <- c(LIST$lhs[eq.idx],
+                                    LIST$rhs[eq.idx])
+
+                    # TODO: what about exceptions?
+                    if(all(plabels.set %in% con.labels)) {
+                        next
+                    }
+                }
+
+                # 1. echelon pattern
+                nfac <- length(lv.nam.efa)
+                for(f in seq_len(nfac)) {
+                    if(f == 1L) {
+                        next
+                    }
+                    nzero <- (f - 1L)
+                    ind.idx <- which(LIST$op == "=~" &
+                                     LIST$block == b &
+                                     LIST$lhs %in% lv.nam.efa[f])
+                    if(length(ind.idx) < nzero) {
+                        stop("lavaan ERROR: efa factor ", lv.nam.efa[f],
+                             " has not enough indicators for echelon pattern")
+                    }
+
+                    # add these plabels
+                    plabel <- c(plabel, LIST$plabel[ind.idx[seq_len(nzero)]])
+                }
+
+                # 2. covariances constrained to zero (only if oblique rotation)
+                if(!orthogonal.efa) {
+                    # skip if user == 1 (user-override!)
+                    cov.idx <- which(LIST$op == "~~" &
+                                     LIST$block == b &
+                                     LIST$user == 0 &
+                                     LIST$lhs %in% lv.nam.efa &
+                                     LIST$rhs %in% lv.nam.efa &
+                                     LIST$lhs != LIST$rhs)
+                    plabel <- c(plabel, LIST$plabel[cov.idx])
+                }
+
+            } # sets
+        } # blocks
+
+        # add zero constraints
+        nel <- length(plabel)
+        TMP <- list()
+        TMP$lhs    <- plabel
+        TMP$op     <- rep("==", nel)
+        TMP$rhs    <- rep(0,    nel)
+        TMP$block  <- rep(0L,   nel)
+        TMP$user   <- rep(7L,   nel) # special code for EFA == 7
+        TMP$ustart <- rep(as.numeric(NA), nel)
+
+        LIST <- lav_partable_merge(LIST, TMP)
+    }
+
+    # handle effect.coding related equality constraints
+    if(is.logical(effect.coding) && effect.coding) {
+        effect.coding <- c("loadings", "intercepts")
+    } else if(!is.character(effect.coding)) {
+        stop("lavaan ERROR: effect.coding argument must be a character string")
+    }
+    if(any(c("loadings", "intercepts") %in% effect.coding)) {
+        TMP <- list()
+        # for each block
+        nblocks <- lav_partable_nblocks(LIST)
+        for(b in seq_len(nblocks)) {
+            # lv's for this block/set
+            lv.names <- unique(LIST$lhs[ LIST$op == "=~" &
+                                         LIST$block == b ])
+
+            if(length(lv.names) == 0L) {
+                next
+            }
+
+            int.plabel <- character(0L)
+            for(lv in lv.names) {
+
+                # ind.names
+                ind.names <- LIST$rhs[ LIST$op == "=~" &
+                                       LIST$block == b &
+                                       LIST$lhs == lv ]
+
+                if("loadings" %in% effect.coding) {
+                    # factor loadings indicators of this lv
+                    loadings.idx <- which( LIST$op == "=~" &
+                                           LIST$block == b &
+                                           LIST$rhs %in% ind.names &
+                                           LIST$lhs == lv )
+
+                    # all free?
+                    if(length(loadings.idx) > 0L &&
+                       all(LIST$free[ loadings.idx ] > 0L)) {
+                        # add eq constraint
+                        plabel <- LIST$plabel[ loadings.idx ]
+
+                        # Note:  we write them as
+                        # .p1. == 3 - .p2. - .p3.
+                        # instead of
+                        # 3 ==  .p1.+.p2.+.p3.
+                        # as this makes it easier to translate things to 
+                        # JAGS/stan
+
+                        LHS <- plabel[1]
+                        if(length(loadings.idx) > 1L) {
+                            RHS <- paste(length(loadings.idx), "-",
+                                         paste(plabel, collapse = "-"), 
+                                         sep = "")
+                        } else {
+                            RHS <- length(loadings.idx)
+                        }
+
+                        TMP$lhs    <- c(TMP$lhs,    LHS)
+                        TMP$op     <- c(TMP$op,     "==")
+                        TMP$rhs    <- c(TMP$rhs,    RHS)
+                        TMP$block  <- c(TMP$block,  0L)
+                        TMP$user   <- c(TMP$user,   2L)
+                        TMP$ustart <- c(TMP$ustart, as.numeric(NA))
+                    }
+                } # loadings
+
+                if("intercepts" %in% effect.coding) {
+                    # intercepts for indicators of this lv
+                    intercepts.idx <- which( LIST$op == "~1" &
+                                             LIST$block == b &
+                                             LIST$lhs %in% ind.names)
+
+                    # all free?
+                    if(length(intercepts.idx) > 0L &&
+                       all(LIST$free[ intercepts.idx ] > 0L)) {
+                        # 1) add eq constraint
+                        plabel <- LIST$plabel[ intercepts.idx ]
+
+                        LHS <- plabel[1]
+                        if(length(intercepts.idx) > 1L) {
+                            RHS <- paste("0-",
+                                         paste(plabel, collapse = "-"), 
+                                         sep = "")
+                        } else {
+                            RHS <- 0L
+                        }
+
+                        TMP$lhs    <- c(TMP$lhs,    LHS)
+                        TMP$op     <- c(TMP$op,     "==")
+                        TMP$rhs    <- c(TMP$rhs,    RHS)
+                        TMP$block  <- c(TMP$block,  0L)
+                        TMP$user   <- c(TMP$user,   2L)
+                        TMP$ustart <- c(TMP$ustart, as.numeric(NA))
+
+                        # 2) release latent mean
+                        lv.int.idx <- which(  LIST$op == "~1" &
+                                              LIST$block == b &
+                                              LIST$lhs == lv )
+                        # free only if automatically added
+                        if(length(lv.int.idx) > 0L &&
+                           LIST$user[ lv.int.idx ] == 0L) {
+                            LIST$free[ lv.int.idx ] <- max(LIST$free) + 1L
+                        }
+                    }
+                } # intercepts
+
+            } # lv
+        } # blocks
+
+        LIST <- lav_partable_merge(LIST, TMP)
+    }
+
+    # mg.lv.variances
+    if(ngroups > 1L && "mg.lv.variances" %in% effect.coding) {
+
+        TMP <- list()
+
+        lv.names <- unique(LIST$lhs[ LIST$op == "=~" ])
+        group.values <- lav_partable_group_values(LIST)
+
+        for(lv in lv.names) {
+            # factor variances
+            lv.var.idx <- which( LIST$op  == "~~" &
+                                 LIST$lhs == lv &
+                                 LIST$rhs == LIST$lhs &
+                                 LIST$lhs == lv )
+
+            # all free (but the first?)
+            if(length(lv.var.idx) > 0L &&
+               all(LIST$free[ lv.var.idx ][-1] > 0L)) {
+                # 1) add eq constraint
+                plabel <- LIST$plabel[ lv.var.idx ]
+
+                TMP$lhs    <- c(TMP$lhs,  length(lv.var.idx))
+                TMP$op     <- c(TMP$op,   "==")
+                TMP$rhs    <- c(TMP$rhs,  paste(plabel, collapse = "+"))
+                TMP$block  <- c(TMP$block,  0L)
+                TMP$user   <- c(TMP$user,   2L)
+                TMP$ustart <- c(TMP$ustart, as.numeric(NA))
+
+                # 2) free lv variances first group
+                lv.var.g1.idx <- which( LIST$op  == "~~" &
+                                        LIST$group == group.values[1] &
+                                        LIST$lhs == lv &
+                                        LIST$rhs == LIST$lhs &
+                                        LIST$lhs == lv )
+                # free only if automatically added
+                if(length(lv.var.g1.idx) > 0L &&
+                   LIST$user[ lv.var.g1.idx ] == 0L) {
+                    LIST$free[ lv.var.g1.idx ] <- max(LIST$free) + 1L
+                }
+            }
+        } # lv
+
+        LIST <- lav_partable_merge(LIST, TMP)
+    }
+
+
     if(debug) {
         cat("[lavaan DEBUG] lavParTable\n")
         print( as.data.frame(LIST) )
