@@ -1,6 +1,6 @@
 ## NOTE:
 ## round(1.2355, 3) = 1.236
-## but 
+## but
 ## round(1.2345, 3) = 1.234
 ##
 ## perhaps we should add 0.0005 or something to avoid this?
@@ -43,7 +43,7 @@ print.lavaan.list <- function(x, ...) {
 print.lavaan.matrix.symmetric <- function(x, ..., nd=3) {
     # print only lower triangle of a symmetric matrix
     # this function was inspired by the `print.correlation' function
-    # in package nlme 
+    # in package nlme
     y <- x; y <- unclass(y)
     ll <- lower.tri(x, diag=TRUE)
     y[ll] <- format(round(x[ll], digits=nd)); y[!ll] <- ""
@@ -82,20 +82,20 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
 
     # format for numeric values
     num.format  <- paste("%", max(8, nd + 5), ".", nd, "f", sep = "")
-    char.format <- paste("%", max(8, nd + 5), "s", sep="") 
+    char.format <- paste("%", max(8, nd + 5), "s", sep="")
 
     # output sections
-    GSECTIONS <- c("Latent Variables", 
-                   "Composites", 
-                   "Regressions", 
+    GSECTIONS <- c("Latent Variables",
+                   "Composites",
+                   "Regressions",
                    "Covariances",
-                   "Intercepts", 
-                   "Thresholds", 
-                   "Variances", 
+                   "Intercepts",
+                   "Thresholds",
+                   "Variances",
                    "Scales y*",
                    "Group Weight",
                    "R-Square")
-    ASECTIONS <- c("Defined Parameters", 
+    ASECTIONS <- c("Defined Parameters",
                    "Constraints")
 
     # header?
@@ -108,32 +108,38 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
         # 1. information
         # 2. se
         # 3. bootstrap requested/successful draws
-        if(!is.null(x$se)) { 
-            # 1.
-            t0.txt <- sprintf("  %-35s", "Information")
-            tmp.txt <- attr(x, "information")
-            t1.txt <- sprintf("  %15s", paste(toupper(substring(tmp.txt,1,1)),
-                             substring(tmp.txt,2), sep=""))
-            cat(t0.txt, t1.txt, "\n", sep="")
+        if(!is.null(x$se)) {
 
-            # 2.
-            if(attr(x, "information") %in% c("expected", "first.order") ||
-               attr(x, "observed.information") == "h1") { 
-                t0.txt <- sprintf("  %-35s", "Information saturated (h1) model")
-                tmp.txt <- attr(x, "h1.information")
-                t1.txt <- sprintf("  %15s", 
-                                  paste(toupper(substring(tmp.txt,1,1)),
-                                          substring(tmp.txt,2), sep=""))
-                cat(t0.txt, t1.txt, "\n", sep="")
-            }
-            if(attr(x, "information") == "observed") {
-                t0.txt <- sprintf("  %-35s", "Observed information based on")
-                tmp.txt <- attr(x, "observed.information")
-                t1.txt <- sprintf("  %15s", 
+            if(attr(x, "se") != "bootstrap") {
+                # 1.
+                t0.txt <- sprintf("  %-35s", "Information")
+                tmp.txt <- attr(x, "information")
+                t1.txt <- sprintf("  %15s",
                                   paste(toupper(substring(tmp.txt,1,1)),
                                   substring(tmp.txt,2), sep=""))
                 cat(t0.txt, t1.txt, "\n", sep="")
-            }
+
+                # 2.
+                if(attr(x, "information") %in% c("expected", "first.order") ||
+                   attr(x, "observed.information") == "h1") {
+                    t0.txt <- sprintf("  %-35s",
+                                      "Information saturated (h1) model")
+                    tmp.txt <- attr(x, "h1.information")
+                    t1.txt <- sprintf("  %15s",
+                                      paste(toupper(substring(tmp.txt,1,1)),
+                                            substring(tmp.txt,2), sep=""))
+                    cat(t0.txt, t1.txt, "\n", sep="")
+                }
+                if(attr(x, "information") == "observed") {
+                    t0.txt <- sprintf("  %-35s",
+                                      "Observed information based on")
+                    tmp.txt <- attr(x, "observed.information")
+                    t1.txt <- sprintf("  %15s",
+                                      paste(toupper(substring(tmp.txt,1,1)),
+                                      substring(tmp.txt,2), sep=""))
+                    cat(t0.txt, t1.txt, "\n", sep="")
+                }
+            } # no bootstrap
 
             # 3.
             t0.txt <- sprintf("  %-31s", "Standard Errors")
@@ -141,14 +147,14 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
             t1.txt <- sprintf("  %19s", paste(toupper(substring(tmp.txt,1,1)),
                                               substring(tmp.txt,2), sep=""))
             cat(t0.txt, t1.txt, "\n", sep="")
-    
+
             # 4.
             if(attr(x, "se") == "bootstrap" && !is.null(attr(x, "bootstrap"))) {
-                t0.txt <- 
+                t0.txt <-
                     sprintf("  %-40s", "Number of requested bootstrap draws")
                 t1.txt <- sprintf("  %10i", attr(x, "bootstrap"))
                 cat(t0.txt, t1.txt, "\n", sep="")
-                t0.txt <- 
+                t0.txt <-
                     sprintf("  %-40s", "Number of successful bootstrap draws")
                 t1.txt <- sprintf("  %10i", attr(x, "bootstrap.successful"))
                 cat(t0.txt, t1.txt, "\n", sep="")
@@ -181,7 +187,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
     y <- as.data.frame(
            lapply(x, function(x) {
                if(is.numeric(x)) {
-                   sprintf(num.format, x)   
+                   sprintf(num.format, x)
                } else {
                    x
                }
@@ -191,6 +197,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
     # always remove /block/level/group/op/rhs/label/exo columns
     y$op <- y$group <- y$rhs <- y$label <- y$exo <- NULL
     y$block <- y$level <- NULL
+    y$efa <- NULL
 
     # if standardized, remove std.nox column (space reasons only)
     # unless, std.all is already removed
@@ -199,9 +206,9 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
     }
 
     # convert to character matrix
-    m <- as.matrix(format.data.frame(y, na.encode = FALSE, 
+    m <- as.matrix(format.data.frame(y, na.encode = FALSE,
                                          justify = "right"))
-   
+
     # use empty row names
     rownames(m) <- rep("", nrow(m))
 
@@ -307,7 +314,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
       colnames(m)[ colnames(m) == "P(>|z|)"] <- "P(>|t|)"
       colnames(m)[ colnames(m) == "riv"    ] <- "RIV"
     }
-    
+
     # format column names
     colnames(m) <- sprintf(char.format, colnames(m))
 
@@ -352,7 +359,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
             # ov/lv names
             ov.names <- lavNames(x, "ov", block = b)
             lv.names <- lavNames(x, "lv", block = b)
-   
+
             # level header
             if(nlevels > 1L) {
                 level.label <- attr(x, "level.label")
@@ -363,7 +370,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
             # group-specific sections
             for(s in GSECTIONS) {
                 if(s == "Latent Variables") {
-                        row.idx <- which( x$op == "=~" & !x$lhs %in% ov.names & 
+                        row.idx <- which( x$op == "=~" & !x$lhs %in% ov.names &
                                       x$block == b)
                     if(length(row.idx) == 0L) next
                     m[row.idx,1] <- .makeNames(x$rhs[row.idx], x$label[row.idx])
@@ -381,7 +388,8 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
                     if(length(row.idx) == 0L) next
                     # make distinction between residual and plain
                     y.names <- unique( c(lavNames(x, "eqs.y"),
-                                         lavNames(x, "ov.ind")) )
+                                         lavNames(x, "ov.ind"),
+                                         lavNames(x, "lv.ind")) )
                     PREFIX <- rep("", length(row.idx))
                     PREFIX[ x$rhs[row.idx] %in% y.names ] <- "  ."
                     m[row.idx,1] <- .makeNames(x$rhs[row.idx], x$label[row.idx],
@@ -392,7 +400,8 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
                     if(length(row.idx) == 0L) next
                     # make distinction between intercepts and means
                     y.names <- unique( c(lavNames(x, "eqs.y"),
-                                         lavNames(x, "ov.ind")) )
+                                         lavNames(x, "ov.ind"),
+                                         lavNames(x, "lv.ind")) )
                     PREFIX <- rep("", length(row.idx))
                     PREFIX[ x$lhs[row.idx] %in% y.names ] <- "  ."
                         m[row.idx,1] <- .makeNames(x$lhs[row.idx], x$label[row.idx],
@@ -401,7 +410,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
                 } else if(s == "Thresholds") {
                     row.idx <- which(x$op == "|" & x$block == b)
                     if(length(row.idx) == 0L) next
-                    m[row.idx,1] <- .makeNames(paste(x$lhs[row.idx], "|", 
+                    m[row.idx,1] <- .makeNames(paste(x$lhs[row.idx], "|",
                         x$rhs[row.idx], sep=""), x$label[row.idx])
                 } else if(s == "Variances") {
                     row.idx <- which(x$op == "~~" & x$lhs == x$rhs & !x$exo &
@@ -409,7 +418,8 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
                     if(length(row.idx) == 0L) next
                     # make distinction between residual and plain
                     y.names <- unique( c(lavNames(x, "eqs.y"),
-                                         lavNames(x, "ov.ind")) )
+                                         lavNames(x, "ov.ind"),
+                                         lavNames(x, "lv.ind")) )
                     PREFIX <- rep("", length(row.idx))
                     PREFIX[ x$rhs[row.idx] %in% y.names ] <- "  ."
                     m[row.idx,1] <- .makeNames(x$rhs[row.idx], x$label[row.idx],
@@ -429,29 +439,35 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
                 } else {
                         row.idx <- integer(0L)
                 }
-    
+
                 # do we need special formatting for this section?
                 # three types:
                 #  - regular (nothing to do, except row/colnames)
                 #  - R-square
                 #  - Latent Variables (and Composites), Regressions and Covariances
                 #    'bundle' the output per lhs element
-                
+
                 # bundling
-                if(s %in% c("Latent Variables", "Composites", 
+                if(s %in% c("Latent Variables", "Composites",
                             "Regressions", "Covariances")) {
                     nel <- length(row.idx)
                     M <- matrix("", nrow = nel*2, ncol = ncol(m))
                     colnames(M) <- colnames(m)
                     rownames(M) <- rep("", NROW(M))
                     #colnames(M)[1] <- sprintf("%-17s", paste(s, ":", sep = ""))
-                    LHS <- paste(x$lhs[row.idx], x$op[row.idx])
+                    if(is.null(x$efa)) {
+                        LHS <- paste(x$lhs[row.idx], x$op[row.idx])
+                    } else {
+                        LHS <- paste(x$lhs[row.idx], x$op[row.idx],
+                                     x$efa[row.idx])
+                    }
                     lhs.idx <- seq(1, nel*2L, 2L)
                     rhs.idx <- seq(1, nel*2L, 2L) + 1L
                     if(s == "Covariances") {
                         # make distinction between residual and plain
                         y.names <- unique( c(lavNames(x, "eqs.y"),
-                                             lavNames(x, "ov.ind")) )
+                                             lavNames(x, "ov.ind"),
+                                             lavNames(x, "lv.ind")) )
                             PREFIX <- rep("", length(row.idx))
                         PREFIX[ x$lhs[row.idx] %in% y.names ] <- "."
                     } else {
@@ -476,7 +492,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
                     cat("\n", s, ":\n", sep = "")
                         #cat("\n")
                     print(M, quote = FALSE)
-    
+
                 # R-square
                 } else if(s == "R-Square") {
                     M <- m[row.idx,1:2,drop=FALSE]
@@ -486,7 +502,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
                     cat("\n", s, ":\n", sep = "")
                     #cat("\n")
                     print(M, quote = FALSE)
-    
+
                 # Regular
                 } else {
                     #M <- rbind(matrix("", nrow = 1L, ncol = ncol(m)),
@@ -500,7 +516,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
                     print(M, quote = FALSE)
                 }
             } # GSECTIONS
-    
+
         } # levels
 
     } # groups
@@ -515,7 +531,7 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
         } else if(s == "Constraints") {
             row.idx <- which(x$op %in% c("==", "<", ">"))
             if(length(row.idx) == 0) next
-            m[row.idx,1] <- .makeConNames(x$lhs[row.idx], x$op[row.idx], 
+            m[row.idx,1] <- .makeConNames(x$lhs[row.idx], x$op[row.idx],
                                           x$rhs[row.idx], nd = nd)
             m[row.idx,2] <- sprintf(num.format, abs(x$est[row.idx]))
             M <- m[row.idx,1:2,drop=FALSE]
@@ -609,13 +625,13 @@ print.lavaan.parameterEstimates <- function(x, ..., nd = 3L) {
 }
 
 summary.lavaan.fsr <- function(object, ...) {
-    
+
     dotdotdot <- list(...)
     if(!is.null(dotdotdot$nd)) {
         nd <- dotdotdot$nd
     } else {
         nd <- 3L
-    }    
+    }
 
     print.lavaan.fsr(x = object, nd = nd, mm = TRUE, struc = TRUE)
 }
@@ -623,7 +639,7 @@ summary.lavaan.fsr <- function(object, ...) {
 print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
 
     y <- unclass(x)
-    
+
     # print header
     if(!is.null(y$header)) {
         cat(y$header)
@@ -636,7 +652,7 @@ print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
         for(b in seq_len(nblocks)) {
             cat("Measurement block for latent variable(s):",
                 paste(lavNames(y$MM.FIT[[b]], "lv")), "\n")
- 
+
             # fit measures?
             b.options <- lavInspect(y$MM.FIT[[b]], "options")
             if(b.options$test != "none") {
@@ -656,7 +672,7 @@ print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
     if(struc) {
         cat("Structural Part\n")
         cat("\n")
-        #print.lavaan.parameterEstimates(y$PE, ..., nd = nd) 
+        #print.lavaan.parameterEstimates(y$PE, ..., nd = nd)
 
         short.summary(y$STRUC.FIT)
         FIT <- fitMeasures(y$STRUC.FIT, fit.measures="default")
@@ -664,9 +680,10 @@ print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
             print.fit.measures( FIT )
         }
     }
-    PE <- parameterEstimates(y$STRUC.FIT, ci = FALSE, 
+    PE <- parameterEstimates(y$STRUC.FIT, ci = FALSE,
                              remove.eq = FALSE, remove.system.eq = TRUE,
                              remove.ineq = FALSE, remove.def = FALSE,
+                             remove.nonfree = FALSE,
                              add.attributes = TRUE)
     print.lavaan.parameterEstimates(PE, ..., nd = nd)
 
@@ -674,4 +691,58 @@ print.lavaan.fsr <- function(x, ..., nd = 3L, mm = FALSE, struc = FALSE) {
 }
 
 
+# print warnings/errors in a consistent way
+# YR 12 July 2018
+
+lav_txt2message <- function(txt, header = "lavaan WARNING:",
+                            footer = "", txt.width = 70L, shift = 3L) {
+    # make sure we only have a single string
+    txt <- paste(txt, collapse = "")
+
+    # split the txt in little chunks
+    chunks <- strsplit(txt, "\\s+", fixed = FALSE)[[1]]
+
+    # chunk size (number of characters)
+    chunk.size <- nchar(chunks)
+
+    # remove empty chunks (needed?)
+    empty.idx <- which(chunk.size == 0)
+    if(length(empty.idx) > 0L) {
+        chunks <- chunks[-empty.idx]
+        chunk.size <- chunk.size[-empty.idx]
+    }
+
+    # insert "\n" so the txt never gets wider than txt.width
+    num.lines <- floor((sum(chunk.size) + length(chunk.size))/txt.width + 0.5)
+    target <- character(num.lines)
+
+    line.size <- shift
+    line.num <- 1L
+    start.chunk <- 1L
+    end.chunck <- 1L
+    for(ch in seq_len( length(chunks) )) {
+        line.size <- line.size + chunk.size[ch] + 1L
+        if(line.size > txt.width) {
+            end.chunk <- ch - 1L
+            target[line.num] <- paste(c(rep(" ", (shift-1)),
+                                        chunks[ start.chunk:end.chunk ]),
+                                      collapse = " ")
+            line.num <- line.num + 1L
+            start.chunk <- ch
+            line.size <- shift + chunk.size[ch] + 1L
+        }
+    }
+    # last line
+    target[line.num] <- paste(c(rep(" ", (shift-1)),
+                              chunks[ start.chunk:ch ]), collapse = " ")
+
+    body <- paste(target, collapse = "\n")
+    if(nchar(footer) == 0L) {
+       out <- paste(c(header, body), collapse = "\n")
+    } else {
+       out <- paste(c(header, body, footer), collapse = "\n")
+    }
+
+    out
+}
 
